package common

import (
	"bytes"
	"testing"

	"github.com/modern-go/parse"
	"github.com/stretchr/testify/require"
)

func TestGetStringNull(t *testing.T) {
	var testCase = []struct {
		in     []byte
		expect string
	}{
		{
			in:     []byte{0x68, 0x65, 0x6c, 0x6c, 0x6f, 0x20, 0x77, 0x6f, 0x72, 0x6c, 0x64, 0x00},
			expect: "hello world",
		},
		{
			in:     []byte{0x68, 0x65, 0x6c, 0x6c, 0x6f, 0x20, 0x77, 0x6f, 0x72, 0x6c, 0x64, 0x00, 0x68, 0x65, 0x6c, 0x6c, 0x6f, 0x20, 0x77, 0x6f, 0x72, 0x6c, 0x64},
			expect: "hello world",
		},
		{
			in:     []byte{0x00, 0x68, 0x65, 0x6c, 0x6c, 0x6f, 0x20, 0x77, 0x6f, 0x72, 0x6c, 0x64, 0x00, 0x68, 0x65, 0x6c, 0x6c, 0x6f, 0x20, 0x77, 0x6f, 0x72, 0x6c, 0x64},
			expect: "",
		},
		{
			in:     []byte{0x49, 0x6c, 0x6f, 0x76, 0x65, 0x44, 0x69, 0x64, 0x69, 0x00},
			expect: "IloveDidi",
		},
	}
	should := require.New(t)
	for idx, tc := range testCase {
		token, err := parse.NewSource(bytes.NewReader(tc.in), 10)
		should.NoError(err)
		actual := GetStringNull(token)
		should.Equal(tc.expect, actual, "case #%d fail", idx)
	}
}
